﻿#V4
#************************************************************************************************************
#
#                              But : Put Winpe in mode Flat into a new VHD
#
#************************************************************************************************************
Write-host -foreground green "Put Winpe in mode Flat into a new VHD."
Write-host -foreground yellow "Please read the information in the script about creating the BCD file."

<#
La langue : 
Si le premier caractère est un _ alors langue = FR
sinon langue = les 2 premiers caractères du nom du script
#>
$langue       = "FR"
$scriptName = $MyInvocation.MyCommand.Name
"scriptName = $scriptName"
if ($scriptName -like "En*") { $langue="En"}


$vhdpath      = "c:\temp\test.vhd"         # vhdpath      = "C:\VHD\Winpe-20H2-Sandbox.vhd"
$label        = "Winpe-20H2-$langue"       # label        = Winpe-20H2-Sandbox
$vhdsize      = 1Gb                        # vhdsize      = size of the VHD
$SrcBoot_wim  = "c:\temp\boot.wim"         # SrcBoot_wim  = the complete path where to take the file boot.wim
$CreateVhd    = $true                      # create new vhd or mount and format an existing chd
$createPartition = $false                  # case re-use disk : create a partition if disk empty

#mes datas
#fr
$SrcBoot_wim     = "C:\Users\noelb\Documents\MicroWinpeBuilder-20H2-$langue\build\MEDIA\sources\boot.wim"
#slore
#$SrcBoot_wim     = "C:\slore\wimbuilder2-master\_Factory_\target\WIN10XPE\build\boot.wim"
#en
#$SrcBoot_wim     = "C:\Users\noelb\Documents\MicroWinpeBuilder-20H2-En\build\MEDIA\sources\boot.wim"
if( ! (test-path $SrcBoot_wim )){ throw "Can't find $SrcBoot_wim"}

#   disque noir
#$vhdpath        = "E:\mesvhd\20H2\Winpe-20H2.vhd"
#   disque blanc
#$vhdpath        = "E:\MesVhd\Winpe-20H2.vhd"
# disque PC-P702
#fr
$vhdpath         = "C:\vhd\Winpe-20H2-$langue.vhd"
#en
#$vhdpath         = "C:\vhd\Winpe-20H2-en.vhd"

#$label           = "SloreWinpe-20H2" 
#$label           = "Winpe-20H2-en"
 
$CreateVhd       = $false
$createPartition = $true

#region functions tools
function pause ([string] $msg){
    "pause..."
    $option = [System.Management.Automation.Host.ReadKeyOptions]"IncludeKeyUp";
    $null=$host.UI.RawUI.ReadKey($option)
    "$msg"
}
function Test-Admin (){
    $identity = [Security.Principal.WindowsIdentity]::GetCurrent()
    $principal = new-object Security.Principal.WindowsPrincipal $identity
    if ( ! $principal.IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator) )
    {
	    write-host -foreground red "$(get-date -Format HH:mm:ss) : This script needs Adm rights (and don't forget executionpolicy for PS) !"
	    break
    }
}
#endregion functions tools

#************************************************************************************************************
#    get the first free letter of drive
#************************************************************************************************************

$p=Get-Partition|Select-Object -property DriveLetter | ?{$_.DriveLetter -gt "A"}
$FirstFreeLetter =  [char][int](($p.driveletter| Measure-Object -Maximum).maximum +1)

#************************************************************************************************************
#
#    1 - Validate yours datas 
#
#************************************************************************************************************
"vhdpath         = $vhdpath"
"label           = $label"
"vhdsize         = $($vhdsize/1GB)GB"
"FirstFreeLetter = $FirstFreeLetter"
"SrcBoot_wim     = $SrcBoot_wim"
"CreateVhd       = $CreateVhd"
"createPartition = $createPartition"

Write-host -foreground "yellow" "Change the datas in this script by yours settings and comment the first return with '#'"
pause "..."
#return


# if boot.wim does not exist then nexit
if( ! (test-path $SrcBoot_wim) ){ break }

# if not admin then exit
Test-Admin

#************************************************************************************************************
#  Warning if VHD attached !
#************************************************************************************************************
$vhd = get-vhd  $vhdpath -ErrorAction SilentlyContinue
if ($vhd.Attached) { 
    write-host -foreground red "$(get-date -Format HH:mm:ss) : VHD attached : confirm and dismount ! "
    write-host -foreground Yellow  "You van use : disMount-VHD -Path $vhdpath"
    return
}


#************************************************************************************************************
#
#    2 - Create a Vhd fixed, mount, initialize MBR, create one partition, formate, active, assign the first free letter
#
#************************************************************************************************************

if ($CreateVhd){

    #************************************************************************************************************
    #delete VHD file it exists : Warning !
    #************************************************************************************************************
    $ErrorActionPreference = "stop"         # for try/catch
    try{
        if (test-path $vhdpath){"`t`tDelete $vhdpath";remove-item $vhdpath}
    }
    catch{
        Write-Warning -Message "$($_.Exception.Message)"
        break                               # quitte the script
    }

    #***********************************************************************************************************
    # Create a Vhd
    #************************************************************************************************************

    write-host -ForegroundColor "cyan" "When the partition will be created, Explorer opens a windows.`t`nPlease focus on this powershell window"
    "$(get-date -Format HH:mm:ss) : Create VHD..."

    $partitionVhd=New-VHD -Path $vhdpath -Fixed -SizeBytes $vhdsize | Mount-VHD -Passthru | Initialize-Disk -PartitionStyle MBR -Passthru | New-Partition  -UseMaximumSize
    if ($partitionVhd -eq $null) {"Error when creating VHD.";return}

}else{

    #***********************************************************************************************************
    # Re-use an existing Vhd with only one partition !
    # exclude GPT : to be sure to not format my disk !
    #************************************************************************************************************

    if ( !(test-path $vhdpath)){ write-host -foreground red "$(get-date -Format HH:mm:ss) : Can't find $vhdpath." ; return }
    Mount-VHD $vhdpath -NoDriveLetter
    $vhd = get-vhd  $vhdpath
    $disk = get-disk -Number $vhd.Number
    #if disque gpt then exit
    if ( -not ($disk.PartitionStyle -ne "MBR" -or $disk.PartitionStyle -ne "RAW")){
        write-host -ForegroundColor red "Disque  $($disk.PartitionStyle)!" ; return
    }
    $nbPart=$disk.NumberOfPartitions
    # if more partition than 1 then exit
    if ($nbPart -gt 1){
        write-host -ForegroundColor red "More than one partition !"
        write-host -ForegroundColor Yellow "Please, verify ! `r`n(When you are sure, you can do : clear-disk -Number $($disk.Number)  -RemoveData -RemoveOEM -Confirm:`$false"
        return
    }
    # if 0 partition then clear-disk ....
    if ($nbPart -eq 0){
        write-host -ForegroundColor cyan "No partition !"
        if ($createPartition){
            write-host -ForegroundColor cyan "Create a partition (clear-disk, Initialize-Disk, New-Partition) ?"
            pause "clear-disk..."
            clear-disk -Number $disk.Number  -RemoveData -RemoveOEM -Confirm:$false
            $partitionVhd = Initialize-Disk -Number  $vhd.Number -PartitionStyle MBR -Passthru | New-Partition  -UseMaximumSize
        }else{ return }
    }
    if ($nbPart -eq 1) {
        write-host -ForegroundColor Yellow "One partition"
        $partitionVhd =  get-partition -DiskNumber $vhd.Number
    }

    "Label actuel : $(($partitionVhd | get-volume ).FileSystemLabel)"
    #confirm
    "ok to Format-Volume ?"
    #pause "Format-Volume..."
    "Format-Volume... (hit once 'enter' after an explorer windows is just opened) "
}

#************************************************************************************************************
#   format
#************************************************************************************************************

$partitionVhd|Format-Volume -FileSystem NTFS -Force -NewFileSystemLabel $label -Confirm:$false
$partitionVhd|Set-Partition -IsActive $true

# I prefer to search again because you could insert a USB drive during the long period of creating VHD
$p=Get-Partition|Select-Object -property DriveLetter | ?{$_.DriveLetter -gt "A"}
$FirstFreeLetter =  [char][int](($p.driveletter| Measure-Object -Maximum).maximum +1)

$partitionVhd|Set-Partition -NewDriveLetter $FirstFreeLetter

#confirm
pause "DISM.exe /Apply-Image..."
 

# now, explorer opens a window -> close it OR set focus to this powershell's windows

#************************************************************************************************************
#    variables for dism and bcdboot
#************************************************************************************************************
$DriveLetter = "$FirstFreeLetter`:"
$driveDst = "$DriveLetter\"

#************************************************************************************************************
#
#    3 - apply boot.wim into VHD
#
#************************************************************************************************************
write-host  -ForegroundColor Green "$(get-date -Format HH:mm:ss) : copy/apply boot.wim into the VHD."

DISM.exe /Apply-Image /Index:1 /ImageFile:$SrcBoot_wim /ApplyDir:$driveDst

#************************************************************************************************************
#
#    3-bis - recopie du fichier fbwf.sys d'origine car le FBWF est inutile dans un VHD
#    cela évite d'avoir le paramètre TestSigning dans le BCD et la neutralisation du "Secure Boot" en UEFI
#
#************************************************************************************************************
write-host -ForegroundColor Green "$(get-date -Format HH:mm:ss) : recopie du fichier fbwf.sys d'origine."
$fbwfSys    = join-path $driveDst "windows\system32\drivers\fbwf.sys"
$fbwfSysOrg = "$fbwfSys.org"
if ( (test-path $fbwfSysOrg) -and (test-path $fbwfSys)){
   Rename-Item  $fbwfSys   "$fbwfSys.mod"
   Copy-Item $fbwfSysOrg $fbwfSys  
}else{
    write-host -ForegroundColor Yellow "fichier absent : $fbwfSysOrg"
}

#************************************************************************************************************
#
#    4 - create BCD with bcdboot
#
#************************************************************************************************************
<#
It is possible to use an VHD in two ways:
- In a VM (I use only hyperV)
    The VHD file is the VM's hard drive
    The BCD file to be created is located inside the VHD
- In a physic machine
    Depending on your needs and tests, the VHD file is contained:
    - either in the machine's hard drive
    - either in an extended USB hard drive
    The BCD file to be modified is contained in the "physical" start-up hard drive
    This script does not deal with this case: it's up to you to make the change
#>

write-host -ForegroundColor Green  "$(get-date -Format HH:mm:ss) : Create BCD for a VM"
bcdboot.exe  $DriveLetter\windows /s $DriveLetter /f ALL

# you can customise the BCD with bcdedit : 
# bootlog = create the list of drivers loaded at boot time, in x:\windows\ntboot.log
bcdedit.exe /store $driveDst\boot\bcd /set `{default`} bootlog YES

#************************************************************************************************************
#
#    5 - Verify in BCD if winpe = Yes
#
#************************************************************************************************************
$w=bcdedit.exe /store $driveDst\boot\bcd
$w
if ( -not ([bool]($w -like "*winpe*"))){ Write-Host -ForegroundColor red "Verify BCD because it seems 'winpe' is absent"}

#************************************************************************************************************
#
#    6 - unmount VHD
#
#************************************************************************************************************
write-host -ForegroundColor Green  "$(get-date -Format HH:mm:ss) Unmount vhd"
disMount-VHD -Path $vhdpath

#************************************************************************************************************
#
#    7 - copy the VHD on a USB drive and modify its BCD ( bcdboot ans bcdedit )
#
#************************************************************************************************************
Write-host -foreground "yellow" "If need, copy the VHD on a USB drive and modify its BCD ( easy with bcdboot and bcdedit )."
